/*
   The function MaxSubArray() is a solution to the maximum-subarray
   problem from Section 4.1 of CLRS.

   This version uses in/out parameters to return the low and high
   indexes of the subarray to the calling function. The sum of the
   subarray is the return value of the function.

   This solution runs in linearithmic time, that is, its O( n*ln(n) ).
*/
#include <stdlib.h>
#include <limits.h>

int FindMaxCrossing(int* a, int* low, int* mid, int* high)
{
   int sum, leftSum, rightSum, maxLeft, maxRight;
   int i, j;

   leftSum = INT_MIN;
   sum = 0;
   for (i = *mid; i >= *low; i--)
   {
      sum += a[i];
      if ( sum > leftSum )
      {
         leftSum = sum;
         maxLeft = i;
      }
   }

   rightSum = INT_MIN;
   sum = 0;
   for (j = *mid + 1; j <= *high; j++)
   {
      sum += a[j];
      if ( sum > rightSum )
      {
         rightSum = sum;
         maxRight = j;
      }
   }

   *low = maxLeft;   // use low and high as "output" parameters
   *high = maxRight;
   return leftSum + rightSum;
}//FindMaxCrossing()



/*
   low and high are both input and output parameters
*/
int MaxSubArray(int* a, int* low, int* high)
{
   int cutoff; //index separating the left and right parts of the input array
   int leftSum, leftLow, leftHigh;
   int rightSum, rightLow, rightHigh;
   int crossSum, crossLow, crossHigh;

   // check for a trivial array
   if ( *low == *high )
   {
      return a[*low];
   }

   cutoff = ((*low)+(*high)-1)/2; // when the array has odd length, the right part is larger

   // recursively solve the sub-problems
   leftLow = *low;
   leftHigh = cutoff;
   leftSum  = MaxSubArray(a, &leftLow, &leftHigh);   // solve the left part
   rightLow = cutoff+1;
   rightHigh = *high;
   rightSum = MaxSubArray(a, &rightLow, &rightHigh); // solve the right part

   // now find the "crossing" subarray
   crossLow = *low;
   crossHigh = *high;
   crossSum = FindMaxCrossing(a, &crossLow, &cutoff, &crossHigh);

   // now determine which of the above three results is the maximumm subarray
   if ( (leftSum >= rightSum) && (leftSum >= crossSum) )
   {
      *low  = leftLow;   // use low and high as "output" parameters
      *high = leftHigh;
      return leftSum;
   }
   else if ( (rightSum >= leftSum) && (rightSum >= crossSum) )
   {
      *low  = rightLow;   // use low and high as "output" parameters
      *high = rightHigh;
      return rightSum;
   }
   else
   {
      *low  = crossLow;   // use low and high as "output" parameters
      *high = crossHigh;
      return crossSum;
   }
}//MaxSubArray()
