/*
   The function MaxSubArray() is a solution to the maximum-subarray
   problem from Section 4.1 of CLRS.

   This solution runs in linearithmic time, that is, its O( n*ln(n) ).
*/
#include <stdlib.h>
#include <limits.h>


typedef struct{
   int sum;
   int low;
   int high;
} ReturnType;


ReturnType* FindMaxCrossing(int* a, int low, int mid, int high)
{
   ReturnType * returnValue;
   int sum, leftSum, rightSum, maxLeft, maxRight;
   int i, j;

   leftSum = INT_MIN;
   sum = 0;
   for (i = mid; i >= low; i--)
   {
      sum += a[i];
      if ( sum > leftSum )
      {
         leftSum = sum;
         maxLeft = i;
      }
   }

   rightSum = INT_MIN;
   sum = 0;
   for (j = mid + 1; j <= high; j++)
   {
      sum += a[j];
      if ( sum > rightSum )
      {
         rightSum = sum;
         maxRight = j;
      }
   }

   // allocate, populate, and return a value
   returnValue = (ReturnType*)malloc( sizeof(ReturnType) );
   returnValue->low = maxLeft;
   returnValue->high = maxRight;
   returnValue->sum = leftSum + rightSum;
   return returnValue;
}//FindMaxCrossing()



ReturnType* MaxSubArray(int* a, int low, int high)
{
   int cutoff; //index separating the left and right parts of the input array

   ReturnType * returnValue;
   ReturnType * leftValue;
   ReturnType * rightValue;
   ReturnType * crossValue;

   // check for a trivial array
   if ( low == high )
   {  // allocate, populate, and return a value
      returnValue = (ReturnType*)malloc( sizeof(ReturnType) );
      returnValue->sum = a[low];
      returnValue->low = low;
      returnValue->high = low;
      return returnValue;
   }

   cutoff = (low+high-1)/2; // when the input array has odd length, the right part is larger

   // recursively solve the sub-problems
   leftValue  = MaxSubArray(a, low,      cutoff); // solve the left part
   rightValue = MaxSubArray(a, cutoff+1, high);   // solve the right part

   // now find the "crossing" subarray
   crossValue = FindMaxCrossing(a, low, cutoff, high);

   // now determine which of the above three results is the maximumm subarray
   if ( (leftValue->sum >= rightValue->sum) && (leftValue->sum >= crossValue->sum) )
   {
      free( crossValue );
      free( rightValue );
      return leftValue;
   }
   else if ( (rightValue->sum >= leftValue->sum) && (rightValue->sum >= crossValue->sum) )
   {
      free( leftValue );
      free( crossValue );
      return rightValue;
   }
   else
   {
      free( leftValue );
      free( rightValue );
      return crossValue;
   }
}//MaxSubArray()
