/*
   The functions in this file test the MaxSubArray function from the include file
   MaxSubArray-linearithmic-ver2.h

   These test functions fill an array of size n, where n is a command line parameter,
   with a random sequence of positive and negative integers, Then the random array
   is searched to find the contiguous subarray with the maximum sum.

   The time it takes to search the array is (roughly) calculated.

   This version handles return values a different way.
*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "MaxSubArray-linearithmic-ver2.h"

/*
   This function prints an array in rows of 10.
*/
void printTheArray(int* a, int p, int q)
{
   int i;
   for (i = p; i <= q; i++)
   {
      printf(" %d", a[i]);
      if ((i+1-p)%10 == 0) printf("\n");
   }
   printf("\n");
}//printTheArray()




int main(int argc, char** argv)
{
   time_t start,end;
   double dif;

   int returnSum, low, high;

   int i;

   int n;
   if (argc > 1)
   {
      n = atoi( argv[1] );  // get parameter from command line
   }
   else
   {
      printf("Usage: MaxSubArray <n>");
      return 0;
   }

   srand( time(NULL) );

   int* integerArray = (int*)malloc(n*sizeof(int));
   if (integerArray == NULL)  // error check
   {
      fprintf(stderr, "MaxSubArray: not enough memory for array of size %d.\n", n);
      return -1;
   }

   // initialize the array with random integers between -1000 and 1000
   for (i = 0; i < n; i++) integerArray[i] = (rand() % 2000) - 1000;

   // if n is small enough, print out the array.
   if (n <= 128)
   {
      printf("Here is the array.\n");
      printTheArray(integerArray, 0, n-1);
   }

   // search the array
   time (&start);
   low = 0;
   high = n-1;
   returnSum = MaxSubArray( integerArray, &low, &high );
   time (&end);

   // as soon as searching is done, print a message.
   printf("\nSearch done.\n");
   dif = difftime (end, start);
   printf ("It took %.2lf seconds to search the array.\n", dif );

   // print out the subarray.
   printf("\nMaxSum = %d, low = %d, high = %d\n", returnSum, low, high);
   if ( high - low <= 128)
   {
      printf("Here is the subarray.\n");
      printTheArray(integerArray, low, high);
   }

   return 0;
}//main()
